package com.qmth.exam.reserve.test;

import com.qmth.boot.tools.signature.SignatureEntity;
import com.qmth.boot.tools.signature.SignatureType;
import com.qmth.exam.reserve.bean.login.LoginUser;
import com.qmth.exam.reserve.util.HttpClientBuilder;
import com.qmth.exam.reserve.util.JsonHelper;
import okhttp3.*;
import org.junit.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class AuthTest {

    private final static String SERVER_URL = "http://localhost:8080";

    // @Test
    public void demo() throws Exception {
        LoginUser loginUser = this.doLogin();
        this.doLogout(loginUser);
    }

    private LoginUser doLogin() {
        String apiUrl = "/api/student/login";
        Map<String, Object> params = new HashMap<>();
        params.put("account", "1500010120019");
        params.put("password", "123456");

        String jsonParams = JsonHelper.toJson(params);
        RequestBody requestBody = FormBody.create(MediaType.parse("application/json;charset=UTF-8"), jsonParams);
        Request.Builder builder = new Request.Builder().post(requestBody).url(SERVER_URL + apiUrl);
        Request request = builder.build();

        try (Response response = HttpClientBuilder.getClient().newCall(request).execute();
             ResponseBody body = response.body();) {
            String bodyStr = body.string();
            System.out.printf("Response:%s %s \n\n", response.code(), bodyStr);

            if (response.isSuccessful()) {
                return JsonHelper.toObj(bodyStr, LoginUser.class);
            }
        } catch (IOException e) {
            System.err.println(e.getMessage());
        }

        throw new RuntimeException("登录失败！");
    }

    private void doLogout(LoginUser loginUser) {
        String apiUrl = "/api/student/logout";
        long timestamp = System.currentTimeMillis();
        String signature = SignatureEntity.build(SignatureType.TOKEN, "POST", apiUrl,
                timestamp, loginUser.getSessionId(), loginUser.getToken());

        Map<String, String> headers = new HashMap<>();
        headers.put("authorization", signature);
        headers.put("time", String.valueOf(timestamp));
        System.out.println(headers);

        RequestBody requestBody = FormBody.create(MediaType.parse("application/json;charset=UTF-8"), "{}");
        Request.Builder builder = new Request.Builder().post(requestBody).url(SERVER_URL + apiUrl);
        for (Map.Entry<String, String> entry : headers.entrySet()) {
            builder.addHeader(entry.getKey(), entry.getValue());
        }
        Request request = builder.build();

        try (Response response = HttpClientBuilder.getClient().newCall(request).execute();
             ResponseBody body = response.body();) {
            String bodyStr = body.string();
            System.out.printf("Response:%s %s \n\n", response.code(), bodyStr);
        } catch (IOException e) {
            System.err.println(e.getMessage());
        }
    }

}
